
/* BEGIN COPYRIGHT NOTICE

Copyright (c) 1992 by John Sarraille and Peter DiFalco
(john@ishi.csustan.edu)

Copyright (c) 1998 by John Sarraille
(john@ishi.csustan.edu)

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that copyright notice and this permission
notice appear in supporting documentation.

The algorithm used in this program was inspired by the paper
entitled "A Fast Algorithm To Determine Fractal Dimensions By
Box Counting", which was written by Liebovitch and Toth, and
which appeared in the journal "Physics Letters A", volume 141,
pp 386-390, (1989).

This program is not warranted: use at your own risk.

END COPYRIGHT NOTICE */


/*
   FD3 PROGRAM FILE

Filename:  coversm.c

OUTLINE OF CALLS:

  mcovSweep +
    FrontIndex
    RearIndex
    GetTotDistPts +
    InitStatRAM 
    GetMinCovStats +
      TraverseNhood +
      UpdateMcovStats +
    FinMinStats +
*/

#include "fd.h"

/* 
    THE FUNCTIONS IN THIS FILE: 
*/
void  mcovSweep       () ;
void  GetTotDistPts   () ;
void  GetMinCovStats  () ;
void  TraverseNhood   () ;
void  UpdateMcovStats () ;
void  FinMinStats     () ;  


/*   
      THE FUNCTIONS NOT IN THIS FILE,
      BUT CALLED FROM HERE:
*/
extern uLong    FrontIndex  () ;     /*  in queue.c   */
extern uLong    RearIndex   () ;     /*  in queue.c   */
extern void  InitStatRAM    () ;     /*  in covers.c  */

/* ######################################################### */
/*                        mcovSweep                          */
/* ######################################################### */
/*
    THIS IS THE FUNCTION THAT GETS THE COVERING STATISTICS WHEN
    THE "mincover" METHOD (-m OPTION) IS USED.  WE ASSUME A
    NON-EMPTY LIST OF POINTS.
*/
void mcovSweep(rDptr, covDptr) 
      ramDataRepType * rDptr ;
      coverDataType  * covDptr  ;
{
  uLong * list = (rDptr->pointList)[0] ;
  uLong * next = (rDptr->nextAfter) ;
  uLong   strtIdx = FrontIndex(&((rDptr->Q)[0])) ;
  uLong   endIdx = RearIndex(&((rDptr->Q)[0])) ;
  uLong   epsIdx, numEps = (covDptr->numCellSizes) ;
    /*
       FIND OUT HOW MANY DISTINGUISHABLE POINTS WE HAVE IN THE
       DATA SET.
    */
  GetTotDistPts(list, next, strtIdx, endIdx, covDptr) ;
    /*
       INITIALIZE THE ARRAYS THAT HOLD STATISTICS FOR EACH CELL SIZE.
    */
  InitStatRAM (covDptr) ;
   /*
      DETERMINE THE STATISTICS FOR FOR EACH CELL SIZE.
   */
  for (epsIdx = 0; epsIdx < numEps; epsIdx++)
    GetMinCovStats(list, next, strtIdx, endIdx, covDptr, epsIdx) ;
  FinMinStats(covDptr) ;  
  return ;
}
/* ######################################################### */
/*                       GetTotDistPts                       */
/* ######################################################### */
/*
    GET THE TOTAL NUMBER OF DISTINGUISHABLE POINTS IN THE DATA
    SET.  ONE COULD ARGUE THAT WE OUGHT TO FIGURED THIS NUMBER
    OUT EARLIER IN THE PROCESSING, FOR EXAMPLE WHEN THE RAM DATA
    REPRESENTATION IS FIRST CREATED. WHEN WE WROTE THE FIRST
    VERSIONS OF THE PROGRAM, WE FELT IT MIGHT SLOW US DOWN TO
    DEVOTE A PASS THROUGH THE DATA TO COMPUTING THE NUMBER OF
    DISTINGUISHABLE POINTS.  AT THAT TIME WE WERE USING THE
    LIEBOVITCH-TOTH ALGORITHM EXCLUSIVELY.  WE ARE WRITING
    GetTotDistPts JUST TO BE USED WITH THE MINCOVER (-m)
    OPTION.  
*/
void GetTotDistPts(list, next, strtIdx, endIdx, covDptr) 
      uLong          * list        ;
      uLong          * next        ;
      uLong            strtIdx ;
      uLong            endIdx  ;
      coverDataType  * covDptr  ;
{
  uLong   curIdx = strtIdx ;
  uLong   tdpc = 1 ; 
  
  while (curIdx != endIdx)
  {
    if ( list[curIdx] != list[next[curIdx]] ) tdpc++ ;
    curIdx = next[curIdx] ;
  }
  (covDptr->totDistPts) = tdpc ;
  return ;
}
/* ######################################################### */
/*                      GetMinCovStats                       */
/* ######################################################### */
/*
    THIS FUNCTION GETS THE COVERING STATISTICS FOR ONE
    PARTICULAR VALUE OF THE CELL SIZE WHEN THE "mincover"
    METHOD (-m OPTION) IS USED.  WE ASSUME A NON-EMPTY LIST OF
    POINTS.
*/
void GetMinCovStats (ls, nx, strtIdx, endIdx, covDptr, epsIdx) 
      uLong          * ls       ;
      uLong          * nx       ;
      uLong            strtIdx  ;
      uLong            endIdx   ;
      coverDataType  * covDptr  ;
      uLong            epsIdx   ;
{
  uLong   lastCellIdx ;
     /*
         dpc = DISTINGUISHABLE POINT COUNT -- TO COUNT THE NUMBER
         OF DISTINGUISHABLE POINTS IN THE CURRENT CELL.  tdpc
         COUNTS THE TOTAL NUMBER OF DISTINGUISHABLE POINTS IN
         THE LIST.
     */
  uLong dpc, tdpc = (covDptr->totDistPts) ;
  int   endOfList ;
     /* LOG (BASE 2) OF THE CELL SIZE. */
  double logCS = (covDptr->logCellSize[epsIdx]) ;
  double ln2 = (covDptr->log2) ;
  double size = exp(logCS * ln2) ;
      /*
          BECAUSE OF "ROUND-OFF ERROR" OUR CALULATED VALUE OF
          THE LARGEST CELL SIZE MIGHT BE A LITTLE "OFF".
          THEREFORE WE JUST SET IT TO THE VALUE IT IS SUPPOSED
          TO HAVE.  THIS WILL INSURE THAT THE CELL COUNT FOR
          THE LARGEST CELL SIZE IS ALWAYS 1 (WE HOPE :-)
      */
  if (epsIdx == (covDptr->numCellSizes) - 1 )
     size = (double)covDptr->maxCoord ;

  TraverseNhood(ls, nx, strtIdx, endIdx, size, &lastCellIdx, &dpc) ;
  UpdateMcovStats (covDptr, epsIdx, dpc, tdpc) ;
  endOfList = (lastCellIdx == endIdx) ;
  while (!endOfList)
  {
    TraverseNhood(ls, nx, nx[lastCellIdx], endIdx, size, &lastCellIdx, &dpc);
    UpdateMcovStats (covDptr, epsIdx, dpc, tdpc) ;
    ((covDptr->cellCount)[epsIdx])++ ;
    endOfList = (lastCellIdx == endIdx) ;
  }
  return ;
}
/* ######################################################### */
/*                      TraverseNhood                        */
/* ######################################################### */
/*
   WE ASSUME HERE THAT WE HAVE A NON-EMPTY LIST (ls) WITH ITS
   ASCENDING ORDER GIVEN BY AN INDEX ARRAY (nx). THE CURRENT
   CELL IS THE CLOSED INTERVAL (IN THE SET OF REAL NUMBERS)
   FROM ls[nHdBaseIdx] TO ls[nHdBaseIdx] + size, WHERE
   size IS THE CELL SIZE (DIAMETER) WE ARE WORKING WITH.  IN
   THIS PROCEDURE, WE MOVE DOWN THE LIST OF DATA VALUES (ls)
   FROM THE BASE OF THE INTERVAL UNTIL REACHING A POINT OUTSIDE
   THE CURRENT CELL, OR UNTIL REACHING THE END OF THE LIST.  WE
   ASSUME THAT nHdBaseIdx IS A VALID INDEX OF ONE OF THE LIST
   ELEMENTS.  THUS, WE ARE ASSUMING THAT THERE IS AT LEAST ONE
   POINT IN THE CURRENT CELL.
*/
void TraverseNhood(ls, nx, nHdBaseIdx, endIdx, size, lastInCellPtr, dpcPtr) 
  uLong   * ls          ;  uLong    * nx            ;
  uLong     nHdBaseIdx  ;  uLong      endIdx        ;
  double    size        ;  uLong    * lastInCellPtr ;
  uLong   * dpcPtr      ;
{
  uLong j = nHdBaseIdx ;
     /*
        COUNT THE CURRENT LIST ELEMENT AS A MEMBER OF THE CURRENT CELL.
     */
  (* dpcPtr) = 1 ;
     /*
        IF THERE IS A NEXT ELEMENT OF THE LIST, AND IF IT LIES
        IN THE CURRENT CELL, THEN WE WILL GO ON TO PROCESS IT.
        IN THE TEST BELOW, j IS ALWAYS THE INDEX OF THE LAST
        ARRAY ELEMENT THAT WAS PROCESSED.
     */
  while ((j != endIdx)
           && (ls[nx[j]] <= ls[nHdBaseIdx] + size))
  {
         /*
            INCREMENT THE COUNT TO SHOW THAT ls[nx[j]] IS IN THE
            CURRENT CELL (UNLESS IT IS INDISTINGUISHABLE IN
            VALUE FROM ls[j])
         */
    if (ls[j] != ls[nx[j]]) (* dpcPtr)++ ;
    j = nx[j] ;
  }
  (* lastInCellPtr) = j ;
  return ;
}
/* ######################################################### */
/*                      UpdateMcovStats                      */
/* ######################################################### */
/*
    UNDER THE mincover (-m) OPTION, THIS PROCEDURE IS CALLED TO
    INCREMENT THE INFORMATION AND SUM-SQUARE-FREQUENCY
    STATISTICS IN ACCORDANCE WITH THE NUMBER OF POINTS FOUND IN
    EACH CELL.  THESE STATISTICS ARE NEEDED TO CALCULATE THE
    INFORMATION DIMENSION AND THE CORRELATION DIMENSION.
*/
void  UpdateMcovStats (covDptr, epsIdx, dpc, tdpc) 
      coverDataType  * covDptr  ;
      uLong            dpc      ;
      uLong            tdpc     ;
      uLong              epsIdx ;
{
  double sProb = dpc/((double) tdpc) ;
  double ln2 = (covDptr->log2) ;

  (covDptr->sumSqrFreq)[epsIdx] += sProb * sProb ;
  (covDptr->information)[epsIdx] += sProb * log(sProb) / ln2 ;
}
/* ######################################################### */
/*                       FinMinStats                         */
/* ######################################################### */
/*
   THIS PROCEDURE CALCULATES NEGATIVE LOGS OF CELL COUNTS AND
   LOGS OF THE SUMS OF THE SQUARE FREQUENCIES.  IT IS TO BE
   CALLED AFTER ALL THE COVERS HAVE BEEN CALCULATED IN THE
   MINCOVER ALGORITHM.
*/
void FinMinStats(covDptr)
        coverDataType  * covDptr  ;
{
  uLong   sizeIdx ;
  double  dTemp ;

  for (sizeIdx = 0; sizeIdx < (covDptr->numCellSizes); sizeIdx++)
  {
    dTemp = (double) ((covDptr->cellCount)[sizeIdx]) ;
    dTemp = -log(dTemp)/(covDptr->log2) ;
    (covDptr->negLogCellCount)[sizeIdx] = dTemp ;

    dTemp = log((covDptr->sumSqrFreq)[sizeIdx]) ;
    dTemp = dTemp/(covDptr->log2) ;
    (covDptr->logSumSqrFreq)[sizeIdx] = dTemp ;
  }
  return ;
}
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */     
/*                      end coversm.c                        */
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */
