
/* BEGIN COPYRIGHT NOTICE

Copyright (c) 1992 by John Sarraille and Peter DiFalco
(john@ishi.csustan.edu)

Copyright (c) 1998 by John Sarraille
(john@ishi.csustan.edu)

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that copyright notice and this permission
notice appear in supporting documentation.

The algorithm used in this program was inspired by the paper
entitled "A Fast Algorithm To Determine Fractal Dimensions By
Box Counting", which was written by Liebovitch and Toth, and
which appeared in the journal "Physics Letters A", volume 141,
pp 386-390, (1989).

This program is not warranted: use at your own risk.

END COPYRIGHT NOTICE */


/*

   FD3 MAIN PROGRAM

Filename:  fdmain.c

Program Usage Summary:

<<program name>> [ -cdfstv ] [-p sat fine] <<input file name>>
<<program name>> [ -cdmstv ] [-p sat fine] <<input file name>>
<<program name>> [ -cdstv ] [argoption ...] <<input file name>>

possible argoptions are: [-m [mag sizes]], or [-p sat fine].

OUTLINE OF CALLS:

main +
  GetMachineRep +
  GetCommand
  PrnUsageMsg +
  PrintCommand  
  PrintMachineRep +
  DoCommand

*/

# include "fd.h"

/* 
     THE FUNCTIONS IN THIS FILE: 
*/
void   main             () ;
void   GetMachineRep    () ;
void   PrnUsageMsg      () ;
void   PrintMachineRep  () ;
  
/*   
     THE FUNCTIONS NOT IN THIS FILE, BUT CALLED FROM HERE:
*/

extern  int   GetCommand        () ;  /* in getcom.c */
extern void   PrintCommand      () ;  /* in getcom.c */
extern void   DoCommand         () ;  /* in docom.c */

/* ######################################################### */
/*                          main                             */
/* ######################################################### */
void main(argc,argv)
        int     argc;
        char    *argv[];
{
   commandType  command ;
   machineType  machine ;
   int          syntaxOK ;

   GetMachineRep (&machine) ;
   syntaxOK = GetCommand (argc, argv, &command, &machine) ;
   if    (!syntaxOK)  PrnUsageMsg (argv, &machine) ;

   if ( (command.debugging) || (command.checking) )
      {
        PrintCommand (argc, argv, &command) ;
        PrintMachineRep (&machine) ;
      }
      
   if (syntaxOK) DoCommand (&command, &machine) ;
   return ;
/*   exit(EXIT_SUCCESS) ; */
}
/* ######################################################### */
/*                      PrnUsageMsg                          */
/* ######################################################### */
void PrnUsageMsg (argv, machPtr)
       char        * argv[] ;
       machineType * machPtr ;
{
   uLong max = machPtr->maxUlong ;
   printf("\n");
   printf("USAGE GUIDE:\n");
   printf("\n");
   printf("%s [ -cdfstv ] [-p sat fine] filename\n", argv[0]);
   printf("%s [ -cdmstv ] [-p sat fine] filename\n", argv[0]);
   printf("%s [ -cdstv ] [-m [mag sizes]] ", argv[0]);
   printf("[-p sat fine] filename\n");
   printf("\n");   
   printf(" (Filename must be no more than") ;
   printf(" %d characters.)\n", MAXfNAMElen) ;
   printf(" (No repeated option flags allowed.)\n") ;
   printf(" (1 < mag < huge.  sizes an integer: 1 < sizes < huge.) \n");
   printf(" suggested value: (log(max)/log(mag) < sizes <= ");
   printf("1+[log(max)/log(mag)])\n");
   printf(" (max = %lu)\n", max);
   printf(" (0 <= sat < fine <= sizes-1.) \n");
   printf("\n");
   fflush(stdout);
   return ;
}
/* ######################################################### */
/*                    GetMachineRep                          */
/* ######################################################### */
/*
      WE CAPTURE A FEW CHARACTERISTICS OF THE "VIRTUAL MACHINE"
      ON WHICH THE PROGRAM IS RUNNING.  

      NOTE HOW WE SET maxUlong=(2^ulongBitNum)-1.  THIS IS THE
      LARGEST VALUE THAT AN UNSIGNED INTEGER CAN HAVE WITH THE
      HOST COMPILER.  WE ARE TRYING TO GET A GOOD VALUE WITHOUT
      RELYING ON ACCESSING A CONSTANT IN A STANDARD INCLUDE
      FILE LIKE <limits.h>.  THIS JOB HAS TO BE DONE CAREFULLY
      TO AVERT AN OVERFLOW.  WE SIMPLY ADD ALL THE INTEGERS
      THAT HAVE A 1 IN ONE BIT POSITION AND 0 IN ALL OTHER
      POSITIONS.  THE RESULT IS AN INTEGER THAT HAS A 1 IN EVERY
      BIT POSITION.

      USING THE SAME TECHNIQUE, WE ALSO SET maxInt =
      (2^(intBitNum-1))-1 -- THIS IS THE LARGEST VALUE THAT AN
      INT CAN HAVE WITH THE HOST COMPILER.  UNLIKE THE UNSIGNED
      CASE ABOVE, WE DO NOT USE THE MOST SIGNIFICANT BIT
      POSITION, SINCE THIS IS RESERVED FOR THE SIGN-BIT.

      WE JUST GET THE VALUE OF THE CONSTANT "DBL_MAX" IN ORDER
      TO RECORD THE VALUE OF THE LARGEST POSSIBLE DOUBLE.
      DBL_MAX IS SUPPOSED TO BE DEFINED IN floats.h.
*/
void GetMachineRep (machinePtr) 
       machineType *  machinePtr ;
  {
    int bitNum ;
    (machinePtr->intByteNum)     = sizeof (int) ;
    (machinePtr->longByteNum)    = sizeof (long) ;
    (machinePtr->ulongByteNum)   = sizeof (uLong) ;
    (machinePtr->doubleByteNum)  = sizeof (double) ;
    (machinePtr->intBitNum)      = 8 * sizeof (int) ;
    (machinePtr->longBitNum)     = 8 * sizeof (long) ;
    (machinePtr->ulongBitNum)    = 8 * sizeof (uLong) ;
    (machinePtr->doubleBitNum)   = 8 * sizeof (double) ;
    (machinePtr->maxDouble)      = DBL_MAX ;
    (machinePtr->maxUlong)       = 0 ;
    for (bitNum = 0; bitNum < (machinePtr->ulongBitNum); bitNum++)
        (machinePtr->maxUlong) += (1 << bitNum);
    (machinePtr->maxInt) = 0 ;
    for (bitNum = 0; bitNum+1 < (machinePtr->intBitNum); bitNum++)
        (machinePtr->maxInt) +=  (1 << bitNum);
    return ;          
  }
/* ######################################################### */
/*                  PrintMachineRep                          */
/* ######################################################### */
void PrintMachineRep (machinePtr) 
       machineType * machinePtr ;
  {
      printf("\n" ) ;
      printf("The MACHINE REPRESENTATION is: \n\n") ;
      printf ("intByteNum       = %d\n",    machinePtr->intByteNum ) ;
      printf ("intBitNum        = %d\n",    machinePtr->intBitNum) ;
      printf ("maxInt           = %d\n",    machinePtr->maxInt) ;
      printf ("longByteNum      = %d\n",    machinePtr->longByteNum) ;
      printf ("longBitNum       = %d\n",    machinePtr->longBitNum ) ;
      printf ("ulongByteNum     = %d\n",    machinePtr->ulongByteNum) ;
      printf ("ulongBitNum      = %d\n",    machinePtr->ulongBitNum) ;
      printf ("maxUlong         = %lu\n",   machinePtr->maxUlong) ;
      printf ("doubleByteNum    = %d\n",    machinePtr->doubleByteNum) ;
      printf ("doubleBitNum     = %d\n",    machinePtr->doubleBitNum) ;
      printf ("maxDouble        = %.8g\n",  machinePtr->maxDouble);
      printf ("\n") ;
      fflush(stdout);
      return ;
  }
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */     
/*                      end fdmain.c                         */
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */
